//
//  BufferedWriter.m
//  BufferedWriter implementation
//
//  $Id: BufferedWriter.m,v 1.1 2006/03/22 07:12:51 rh Exp $
//
//  Created by Rene Hexel on 22/03/06.
//  Copyright 2006 Rene Hexel. All rights reserved.
//

#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#import "BufferedWriter.h"


@implementation BufferedWriter

/*
 * convenience method that returns an autoreleased BufferedWriter instance
 * with standard output as the source
 */
+ (BufferedWriter *) bufferedWriterWithStandardOutput
{
	return [[[BufferedWriter alloc] initWithStandardOutput] autorelease];
}


/*
 * initialise to write to a given file
 */
- initWithFile: (NSString *) fileName
{
	if (!(self = [self init])) return nil;

	fd = open([fileName cString], O_WRONLY | O_CREAT | O_TRUNC, 0644);
	if (fd < 0)
	{
		fprintf(stderr, "Cannot open '%s': %s\n",
			[fileName cString], strerror(errno));
		[self release];
		return nil;
	}

	file = [[NSFileHandle alloc] initWithFileDescriptor: fd
					     closeOnDealloc: YES];
	if (!file)
	{
		close(fd);
		[self release];
		return nil;
	}
	
	return self;
}


/*
 * initialise to write to standard output
 */
- (BufferedWriter *) initWithStandardOutput
{
	if (!(self = [self init])) return nil;
	
	file = [[NSFileHandle fileHandleWithStandardOutput] retain];
	
	if (!file)
	{
		[self release];
		return nil;
	}

	return self;
}


/*
 * close the file (cannot be used afterwards)
 */
- (void) closeFile
{
	[file closeFile];
}


/*
 * release referenced objects when deallocating
 */
- (void) dealloc
{
	[self closeFile];
	[file release];

	[super dealloc];
}


/*
 * write a string to the output file (using UTF 8 encoding)
 */
- (void) writeString: (NSString *) str
{
	[file writeData: [str dataUsingEncoding: NSUTF8StringEncoding]];
}


/*
 * write a string followed by '\n'
 */
- (void) writeLine: (NSString *) line
{
	[self writeString: [NSString stringWithFormat: @"%@\n", line]];
}


@end
